
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/underpants:dist

from setuptools import setup

setup(**{
    'classifiers': [
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries',
    ],
    'description': 'Leverage the Rules Engine from the Pants Build System as a library.',
    'install_requires': (
        'pantsbuild.pants.testutil<2.10,>=2.9.0rc0',
        'pantsbuild.pants<2.10,>=2.9.0rc0',
    ),
    'license': 'MIT',
    'long_description': """
`underpants` - Or, using Pants as an embedded rule engine
=========================================================

DISCLAIMER: This is not in any way to show how to use Pants, or recommended patterns of
use. Provided here AS-IS. No gurantees of any kind, as this may break with any future release of
Pants.

This is a toy POC testing out the feasability of using the [Pants Build
System](https://www.pantsbuild.org/) as a rule engine in your Python application.

""",
    'long_description_content_type': 'text/markdown',
    'name': 'underpants',
    'namespace_packages': (
    ),
    'package_data': {
    },
    'packages': (
        'underpants',
    ),
    'project_urls': {
        'Documentation': 'https://github.com/kaos/underpants/wiki',
        'Source': 'https://github.com/kaos/underpants',
        'Tracker': 'https://github.com/kaos/underpants/issues',
    },
    'version': '0.1.1',
})
